<?php
/*
 +--------------------------------------------------------------------+
 | CiviCRM version 1.6                                                |
 +--------------------------------------------------------------------+
 | copyright CiviCRM LLC (c) 2004-2006                                  |
 +--------------------------------------------------------------------+
 | This file is a part of CiviCRM.                                    |
 |                                                                    |
 | CiviCRM is free software; you can copy, modify, and distribute it  |
 | under the terms of the Affero General Public License Version 1,    |
 | March 2002.                                                        |
 |                                                                    |
 | CiviCRM is distributed in the hope that it will be useful, but     |
 | WITHOUT ANY WARRANTY; without even the implied warranty of         |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.               |
 | See the Affero General Public License for more details.            |
 |                                                                    |
 | You should have received a copy of the Affero General Public       |
 | License along with this program; if not, contact the Social Source |
 | Foundation at info[AT]civicrm[DOT]org.  If you have questions       |
 | about the Affero General Public License or the licensing  of       |
 | of CiviCRM, see the Social Source Foundation CiviCRM license FAQ   |
 | http://www.civicrm.org/licensing/                                  |
 +--------------------------------------------------------------------+
*/

/**
 *
 * @package CRM
 * @author Donald A. Lobo <lobo@civicrm.org>
 * @copyright CiviCRM LLC (c) 2004-2006
 * $Id$
 *
 */



require_once 'CRM/Profile/Form.php';
require_once 'CRM/Contribute/Form/Task.php';

/**
 * This class provides the functionality for batch profile update for contribution
 */
class CRM_Contribute_Form_Task_PickProfile extends CRM_Contribute_Form_Task {

    /**
     * the title of the group
     *
     * @var string
     */
    var $_title;

    /**
     * maximum contributions that should be allowed to update
     *
     */
    var $_maxContributions = 100;

    /**
     * maximum profile fields that will be displayed
     *
     */
    var $_maxFields = 9;


    /**
     * variable to store redirect path
     *
     */
    var $_userContext;


    /**
     * build all the data structures needed to build the form
     *
     * @return void
     * @access public
     */
    function preProcess( ) 
    {
        /*
         * initialize the task and row fields
         */
        parent::preProcess( );
        $session =& CRM_Core_Session::singleton();
        $this->_userContext = $session->readUserContext( );
    
        $validate = false;
        //validations
        if ( count($this->_contributionIds) > $this->_maxContributions) {
            CRM_Core_Session::setStatus("The maximum number of contributions you can select for Batch Update is {$this->_maxContributions}. You have selected ". count($this->_contributionIds). ". Please select fewer contributions from your search results and try again." );
            $validate = true;
        }
        
        if ($validate) { // than redirect
            CRM_Utils_System::redirect( $this->_userContext );
        }
    }
  
    /**
     * Build the form
     *
     * @access public
     * @return void
     */
    function buildQuickForm( ) 
    {
        CRM_Utils_System::setTitle( ts('Batch Profile Update for Contribution') );
        // add select for groups
        $profiles = array( '' => ts('- select profile -')) + CRM_Core_BAO_UFGroup::getProfiles(array('Contribution'));

        $ufGroupElement = $this->add('select', 'uf_group_id', ts('Select Profile'), $profiles, true);
        $this->addDefaultButtons( ts( 'Continue >>' ) );
    }

    /**
     * Add local and global form rules
     *
     * @access protected
     * @return void
     */
    function addRules( ) 
    {
        $this->addFormRule( array( 'CRM_Contribute_Form_Task_PickProfile', 'formRule' ) );
    }
    
    /**
     * global validation rules for the form
     *
     * @param array $fields posted values of the form
     *
     * @return array list of errors to be posted back to the form
     * @static
     * @access public
     */
     function formRule( &$fields ) 
    {
        return true;
    }    

    /**
     * process the form after the input has been submitted and validated
     *
     * @access public
     * @return None
     */
     function postProcess() 
    {
        $params = $this->exportValues( );
        $this->set( 'ufGroupId', $params['uf_group_id'] );
    }//end of function
}
?>
